#include "mainwidget.h"
#include "ui_mainwidget.h"
#include <QDebug>

#define PRM_ID "id"
#define PRM_NAME "name"
#define PRM_FULLNAME "fullname"
#define PRM_CITY_ID "city_id"

MainWidget::MainWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MainWidget)
{
    ui->setupUi(this);

    //Model for table City
    _mCities = new VariantMapTableModel(this);
    _mCities->registerColumn(new SimpleColumn(PRM_ID));
    _mCities->registerColumn(new SimpleColumn(PRM_NAME));

    //Row 1
    QVariantMap city1;
    city1.insert(PRM_ID, 1);
    city1.insert(PRM_NAME, "Paris");
    _mCities->addRow(city1);

    //Row 2
    QVariantMap city2;
    city2.insert(PRM_ID, 2);
    city2.insert(PRM_NAME, "Moscow");
    _mCities->addRow(city2);

    //Row 3
    QVariantMap city3;
    city3.insert(PRM_ID, 3);
    city3.insert(PRM_NAME, "London");
    _mCities->addRow(city3);

    ui->comboBox->setModel(_mCities);
    ui->comboBox->setModelColumn(1);

    //Model for table Contacts
    _mContacts = new VariantMapTableModel(this);
    _mContacts->registerColumn(new SimpleColumn(PRM_ID));
    _mContacts->registerColumn(new SimpleColumn(PRM_FULLNAME));
    _mContacts->registerColumn(new SimpleColumn(PRM_CITY_ID));
    _mContacts->registerColumn(new LookupColumn("city", _mCities, PRM_CITY_ID, PRM_NAME));

    //Contact 1
    QVariantMap contact1;
    contact1.insert(PRM_ID, 1);
    contact1.insert(PRM_FULLNAME, "John Brown");
    contact1.insert(PRM_CITY_ID, 1);
    _mContacts->addRow(contact1);

    //Contact 2
    QVariantMap contact2;
    contact2.insert(PRM_ID, 2);
    contact2.insert(PRM_FULLNAME, "Mary Smith");
    contact2.insert(PRM_CITY_ID, 2);
    _mContacts->addRow(contact2);

    ui->tableView->setModel(_mContacts);
    ui->tableView->hideColumn(2);

    _mapper = new QDataWidgetMapper(this);
    _mapper->setModel(_mContacts);
    _mapper->addMapping(ui->lineEdit, 1);
    _mapper->addMapping(ui->comboBox, 2);
    _mapper->setSubmitPolicy(QDataWidgetMapper::ManualSubmit);

    connect(ui->tableView->selectionModel(), &QItemSelectionModel::currentChanged,
            this, &MainWidget::onContactSelected);
}

MainWidget::~MainWidget()
{
  delete ui;
}

void MainWidget::onContactSelected(const QModelIndex &index)
{
  //Populates editor widgets with selected Contact values
  _mapper->setCurrentIndex(index.row());
}

void MainWidget::on_btnSubmit_clicked()
{
  //Update Contacts model with values from editor widgets
  _mapper->submit();
}
