#ifndef VARIANTMAPTABLEMODEL_H
#define VARIANTMAPTABLEMODEL_H

#include <QAbstractTableModel>
#include <QObject>

#define PRM_ID "id"

//Abstract column class
class AbstractColumn
{
public:
    AbstractColumn(QString name);
    virtual ~AbstractColumn();
    QString name() { return _name; }
    bool editable() { return _editable; }
    virtual QVariant colData(const QVariantMap &rowData, int role = Qt::DisplayRole) = 0;
private:
    QString _name;
protected:
    bool _editable = true;
};

//Default column implementation
class SimpleColumn : public AbstractColumn
{
public:
    SimpleColumn(QString name);

    // AbstractColumn interface
public:
    virtual QVariant colData(const QVariantMap &rowData, int role) override;
};


class VariantMapTableModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    VariantMapTableModel(QObject *parent = nullptr);
    void registerColumn(AbstractColumn* column);
    void addRow(QVariantMap rowData);

    //Convenience methods
    int idByRow(int row) const;
    int rowById(int id) const;
    int colByName(QString name) const;
    QString nameByCol(int col) const;
private:
    //Storing data members
    QList<int> _rowIndex;
    QHash<int, QVariantMap> _dataHash;
    QList<AbstractColumn*> _columns;

    // QAbstractItemModel interface
public:
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
};

class LookupColumn : public AbstractColumn
{
public:
    LookupColumn(QString name, VariantMapTableModel* model, QString keyCol, QString lookupCol);
protected:
    VariantMapTableModel* _lookupModel;
    QString _keyColumn;
    QString _lookupColumn;
    // AbstractColumn interface
public:
    virtual QVariant colData(const QVariantMap &rowData, int role) override;
};

#endif // VARIANTMAPTABLEMODEL_H
